#!/usr/bin/env python
#usage: ./analyze_size_distribution_G_bodies <input_directory_with_segmented_G body images in tab delimited text format>
import sys
import matplotlib as plt
import os
import numpy as np
import scipy.optimize as opt
import pylab as plt

#the twoD gaussian was adapted from Ali_M on stackoverflow: https://stackoverflow.com/questions/21566379/fitting-a-2d-gaussian-function-using-scipy-optimize-curve-fit-valueerror-and-m
#accessed 4/30/19
def twoD_Gaussian((x, y), amplitude, xo, yo, sigma_x, sigma_y, theta, offset):
    xo = float(xo)
    yo = float(yo)    
    a = (np.cos(theta)**2)/(2*sigma_x**2) + (np.sin(theta)**2)/(2*sigma_y**2)
    b = -(np.sin(2*theta))/(4*sigma_x**2) + (np.sin(2*theta))/(4*sigma_y**2)
    c = (np.sin(theta)**2)/(2*sigma_x**2) + (np.cos(theta)**2)/(2*sigma_y**2)
    g = offset + amplitude*np.exp( - (a*((x-xo)**2) + 2*b*(x-xo)*(y-yo) 
                            + c*((y-yo)**2)))
    return g.ravel()

input_files = {}
all_files = []
image_outputs = {}
input_files_data = {}
input_files_arrays = {}

for item in os.listdir(sys.argv[1]):
    if "background" in item:
        n=0
        f = open(sys.argv[1]+item)
        mylist = []
      
        for i, thing in enumerate(f):
            
            if i == 0:

                continue
            else:
                thing = thing.rstrip("\r\n").split("\t")

                mylist.append(int(thing[4]))
                if len(mylist) > 3:
                    del mylist[0]
        n = np.average(mylist)
                
        line = item.rstrip("\_background.txt")
        input_files[line] = [n]
        input_files_data[line] = []
        input_files_arrays[line] = []
        image_outputs[line] = []
    else:
        all_files.append(item)

for item in input_files:
    for files in all_files:
        if 'maxima' in files:
            continue
        elif item in files:
            input_files[item].append(files)
            
sample = []
for item in input_files:
    for n, myfile in enumerate(input_files[item]):
        if n > 0:
            input_files_data[item].append([])
            input_files_arrays[item].append([])
            f = open(sys.argv[1]+myfile)
            myvertline = []
            for i, line in enumerate(f):
                myline = line.rstrip("\r\n").split("\t")

                for m, val in enumerate(myline):
                    if float(val) - input_files[item][0] < 0:
                        myline[m] = 0.0
                    else:
                        myline[m] = float(val)
                input_files_data[item][n-1].append(myline)
            a = np.array(input_files_data[item][n-1])
            sample = a
            input_files_arrays[item][n-1].append(a)

            if 1 == 1:
                continue    
            

x = np.linspace(0,29,30)
y = np.linspace(0,29,30)
x, y = np.meshgrid(x, y)

plt.figure()
plt.imshow(sample.reshape(30, 30))
plt.show()
sample2 = sample.reshape(900,)
initial_guess = (3,1,1,1,1,1,1)
popt, pcov = opt.curve_fit(twoD_Gaussian, (x, y), sample2, p0=initial_guess)
data_fitted = twoD_Gaussian((x, y), *popt)

fig, ax = plt.subplots(1, 1)
ax.hold(True)
ax.imshow(sample2.reshape(30, 30), cmap=plt.cm.jet, origin='bottom',
    extent=(x.min(), x.max(), y.min(), y.max()))
ax.contour(x, y, data_fitted.reshape(30, 30), 8, colors='w')
print popt
plt.show()

myparams = {}
m = 0
errors = 0
successits = 0
for item in input_files_arrays:
    myparams[item] = []
    for mastuff in input_files_arrays[item]:
        for marray in mastuff:
            if marray.shape == (30,30):
                
                marray2 = marray.reshape(900,)
                try:
                    popt, pcov = opt.curve_fit(twoD_Gaussian, (x, y), marray2, p0=initial_guess)

                    if float(abs(popt[3])) > 30.0:
                        print "Impossible sigma X"
                        data_fitted = twoD_Gaussian((x,y), *popt)
                        fig, ax = plt.subplots(1, 1)
                        ax.hold(True)
                        ax.imshow(marray2.reshape(30, 30), cmap=plt.cm.jet, origin='bottom',
                            extent=(x.min(), x.max(), y.min(), y.max()))
                        ax.contour(x, y, data_fitted.reshape(30, 30), 8, colors='w')
                        plt.show()
                        errors +=1
                    elif float(abs(popt[4])) > 30.0:
                        print "Impossible sigma Y"
                        data_fitted = twoD_Gaussian((x,y), *popt)
                        fig, ax = plt.subplots(1, 1)
                        ax.hold(True)
                        ax.imshow(marray2.reshape(30, 30), cmap=plt.cm.jet, origin='bottom',
                            extent=(x.min(), x.max(), y.min(), y.max()))
                        ax.contour(x, y, data_fitted.reshape(30, 30), 8, colors='w')
                        plt.show()
                        errors +=1
                    else:
                        myparams[item].append(popt)

                        successits +=1
                except RuntimeError:
                    print ("Error - curve_fit failed ")
                    errors += 1
                m = m+1
                print m
print errors
print successits

for item in myparams:
    print item
    for thing in myparams[item]:
        myprint = []

        for entry in thing:
            myprint.append(str(entry))
        print "\t".join(myprint)